/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
   There are three ways an object can appear larger in an image.
   The object can grow in size (scale up) the object can move
   closer to the camera, or the camera can zoom in on the object.
   If you look at an object without any frame of reference, you
   cannot tell which of these three operations is making the
   object larger (this ambiguity is the basis for many optical
   illusions). But the right frame of reference can make it
   clear which operation is causing the object to get larger.
*/
public class Zoom_vs_Scale_vs_Move
{
   private static double zTranslation = 0.0;
   private static double scale = 1.0;
   private static double zoom = 1.0;
   private static boolean showFloor = false;

   private static double near   =  1.0;

   public static void main(String[] args)
   {
      System.out.println("Use the m/M, key to move the sphere along the z axe.");
      System.out.println("Use the s/S keys to scale the size of the sphere.");
      System.out.println("Use the z/Z keys to zoom the camera in and out of the scene.");
      System.out.println("Use the 'f' key to toggle the display of the floor on and off.");
      System.out.println("Use the 'c' key to change the random ssphere colors.");


      final Model sphere = new SphereModel(30, 30, 1);
      sphere.setColor(Color.red);
      sphere.modelTranslate(0, 0, -5);

      final Model xzPlane = new PanelModel(-6, 6, -30, 6);
      xzPlane.setColor(Color.black);
      xzPlane.modelTranslate(0, -1, -6);


      // Create the Scene object that we shall render
      final Scene scene = new Scene();

      // Add the sphere to the Scene.
      scene.addModel(xzPlane, sphere);  // the order is important


      // Create a Camera object.
      final Camera camera = new Camera();

      // Set up the camera's view volume.
      camera.projPerspective(-1, +1, -1, +1, near);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=1;
      double centerx=0, centery=0, centerz=-1;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Interactive JFrame", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('f' == c)
            {
               showFloor = ! showFloor;
            }
            else if ('m' == c)
            {
               zTranslation += 0.5;
            }
            else if ('M' == c)
            {
               zTranslation -= 0.5;
            }
            else if ('s' == c) // Scale the sphere 5% smaller.
            {
               scale *= 0.95;
            }
            else if ('S' == c) // Scale the sphere 5% larger.
            {
               scale *= 1.05;
            }
            else if ('z' == c)
            {
               near -= 0.1;
            }
            else if ('Z' == c)
            {
               near += 0.1;
            }
            else if ('c' == c)
            {
               // Change the color of the sphere.
               sphere.setColorRandom();
            }

            // Set the model-to-view transformation matrix.
            // The order of the transformations is important!
            sphere.model2Identity();
            sphere.modelTranslate(0, 0, -5 + zTranslation);
            sphere.modelScale(scale, scale, scale);

            // Set up the camera's view volume.
            camera.projPerspective(-1, +1, -1, +1, near);

            if (showFloor)
               xzPlane.setColor(Color.darkGray);
            else
               xzPlane.setColor(Color.black);

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
